//========================================================================
//
// pdftotext.cc
//
// Copyright 1997 Derek B. Noonburg
//
//========================================================================

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <unistd.h>
#include <sys/kernel.h>
#include "../goo/parseargs.h"
#include "../goo/GString.h"
#include "../goo/gmem.h"
#include "Object.h"
#include "Stream.h"
#include "Array.h"
#include "Dict.h"
#include "XRef.h"
#include "Catalog.h"
#include "Page.h"
#include "PDFDoc.h"
#include "TextOutputDev.h"
#include "Params.h"
#include "Error.h"
#include "config.h"

static int firstPage = 1;
static int lastPage = 0;
static GBool useASCII7 = gFalse;
#if JAPANESE_SUPPORT
static GBool useEUCJP = gFalse;
#endif
static GBool rawOrder = gFalse;
GBool printCommands = gFalse;
static GBool printHelp = gFalse;

#define ISFILE 1
#define ISPAGE 2
#define ISCLOSE 3

struct Proto {
	int m_Type;
	char m_FileName [80];
	int m_PageNum; };

static ArgDesc argDesc[] = {
  {"-f",      argInt,      &firstPage,     0,
   "first page to convert"},
  {"-l",      argInt,      &lastPage,      0,
   "last page to convert"},
  {"-ascii7", argFlag,     &useASCII7,     0,
   "convert to 7-bit ASCII (default is 8-bit ISO Latin-1)"},
#if JAPANESE_SUPPORT
  {"-eucjp",  argFlag,     &useEUCJP,      0,
   "convert Japanese text to EUC-JP"},
#endif
  {"-raw",    argFlag,     &rawOrder,      0,
   "keep strings in content stream order"},
  {"-q",      argFlag,     &errQuiet,      0,
   "don't print any messages or errors"},
  {"-h",      argFlag,     &printHelp,     0,
   "print usage information"},
  {"-help",   argFlag,     &printHelp,     0,
   "print usage information"},
  {NULL}
};

int main(int argc, char *argv[]) {
  PDFDoc *doc;
  GString *fileName;
  GString *textFileName;
  TextOutputDev *textOut;
  int pid;
  int done=0;
  int m_size;
  struct Proto rxbuff;
  GBool ok;
  char *p;
  char * curwd;
  char buf [150];

  // parse args
  
  ok = parseArgs(argDesc, &argc, argv);
  if (!ok || argc < 2 || argc > 3 || printHelp) {
    fprintf(stderr, "pdftotext version %s\n", xpdfVersion);
    fprintf(stderr, "%s\n", xpdfCopyright);
    printUsage("pdftotext", "<PDF-file> [<text-file>]", argDesc);
    exit(1);
  }
  //if(!(strcmpi(argv[1],"Continue")))
	//exit (1);

while (done==0) {
//loop until GUI exits
  
  pid=Receive(0,&rxbuff,sizeof(struct Proto));
  if (rxbuff.m_Type==ISFILE){	
	  fileName = new GString(rxbuff.m_FileName);
  

	  // init error file
	  errorInit();

	  // read config file
	  initParams(xpdfConfigFile);

	  // open PDF file
	  xref = NULL;
	  doc = new PDFDoc(fileName);
	  if (!doc->isOk()) {
		goto err1;
	  }

	  // check for copy permission
	  if (!doc->okToCopy()) {
		error(-1, "Copying of text from this document is not allowed.");
		goto err2;
	  }

	  // construct text file name
	  if (argc == 3) {
		textFileName = new GString(argv[2]);
	  } else {
		p = fileName->getCString() + fileName->getLength() - 4;
		if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF"))
		  textFileName = new GString(fileName->getCString(),
					 fileName->getLength() - 4);
		else
		  textFileName = fileName->copy();
		textFileName->append(".txt");
	  }

	  // get page range
	  if (firstPage < 1)
		firstPage = 1;
	  if (lastPage < 1 || lastPage > doc->getNumPages())
		lastPage = doc->getNumPages();

	  // write text file
	#if JAPANESE_SUPPORT
	  useASCII7 |= useEUCJP;
	#endif
	curwd=getcwd(buf,150);
	if (strcmp(curwd,"/"))
		strcat(curwd,"/");
	  textOut = new TextOutputDev(curwd, useASCII7, rawOrder);
	  if (textOut->isOk())
		doc->displayPages(textOut, firstPage, lastPage, 72, 0);
	m_size=textOut->getSize();	
	Reply(pid,&m_size,sizeof(firstPage));
	pid=Receive(0,&rxbuff,sizeof(struct Proto));
	 Reply(pid,&lastPage,sizeof(lastPage));
	} //end of ISFILE
	else if (rxbuff.m_Type==ISPAGE)
	{
		firstPage=rxbuff.m_PageNum;
		doc->displayPages(textOut, firstPage, lastPage, 72, 0);
		m_size=textOut->getSize();	
		Reply(pid,&m_size,sizeof(firstPage));
		pid=Receive(0,&rxbuff,sizeof(struct Proto));
		Reply(pid,&firstPage,sizeof(firstPage));
	}
	else if (rxbuff.m_Type==ISCLOSE)
	{
		delete textOut;
		 // clean up
	    delete textFileName;
		delete doc;
		freeParams();	
		Reply(pid,"QUITTING",9);
	}
}// end of while		
 
 
 
 err2:
  delete doc;
 err1:
  freeParams();

  // check for memory leaks
  Object::memCheck(stderr);
  gMemReport(stderr);
  return 0;
}
